######################################################################
#
# IMAPMessage, manage IMAP messages for Webmail product
#
# This software is governed by a license. See 
# LICENSE.txt for the terms of this license.
#
######################################################################

from IMAPPasserelle import *
from IMAPMessage import *
import cStringIO
import mimetools
import MimeWriter
import mimify
import re
import string
from DocumentTemplate.DT_Util import html_quote
from DocumentTemplate.DT_Var import newline_to_br
import quopri

class IMAPMessage:
    """ IMAP message """


    def __init__(self, IMAPId="", flags={}, expediteur=[], size=0, date="", subject="[pas de sujet]", body="", destinataires={'to':[], 'cc':[], 'bcc':[]}, attachements=[], headers={}):
        """ constructor"""

        self.headers=headers
        self.IMAPId=IMAPId
        self.flags=flags
        self.expediteur=expediteur
        self.size=size
        self.date=date
        self.subject=subject
        self.body=body
        self.destinataires=destinataires
        self.attachements=attachements

    def getIMAPId(self):
        """ return IMAP Id fo the message"""
        return self.IMAPId
    
    def isRead(self):
        """ return the value of flag Seen (= Read) """
        return self.flags['read']

    def isAnswered(self):
        """ return the value of the flag Answered """
        return self.flags['answered']

    def isDeleted(self):
        """ return the value of the flag Deleted """
        return self.flags['deleted']

    def isFlagged(self):
        """ return the value of the flag Flagged """
        return self.flags['flagged']

    def getSize(self):
        """ return the size of the message """
        return self.size

    def getDate(self):
        """ return readable date of the message """
        return self.date

    def getSubject(self):
        """ return the subject """
        return self.subject

    def setSubject(self, subject):
        """ set the subject """

        self.subject=subject

    def getExpediteurName(self):
        """ return the sender name """

        return string.replace(self.expediteur[0],"\"",'')
        #return self.expediteur[0]

    def getExpediteurMail(self):
        """ return the sender mail """

        return string.replace(self.expediteur[1],"\"",'')
        #return self.expediteur[1]

    def getTo(self):
        """ return the 'to' field (list)"""
        return self.destinataires['to']

    def setTo(self, to):
        """ set to value """
        self.destinataires['to']=to

    def getCC(self):
        """ return the 'cc' field (list)"""
        return self.destinataires['cc']

    def setCC(self, cc):
        """ set cc value """
        self.destinataires['cc']=cc

    def getBCC(self):
        """ return the 'bcc' field (list)"""
        return self.destinataires['bcc']

    def setBCC(self, bcc):
        """ set bcc value """
        self.destinataires['bcc']=bcc

    def getBody(self):
        """ return the body of the message """
        return self.body

    def setBody(self, body):
        """ set the message body """

        self.body=body

    def setDestinataires(self, destinataires):
        """ set destinataires values"""
        
        self.destinataires=destinataires

    def getAttachements(self):
        """ return the attachement list"""
        return self.attachements

    def delAttachement(self, id):
        """ delete a attachement """

        #to do

    def setBody(self, body):
        """ set message body """
        self.body = body

    def edit(self, body, subject, date, expediteur, attachement, to, cc, bcc, copy_to_folder, save_draft):
        """ edit a message """
        
        self.setBody(body)
        self.subject = subject
        self.date = date
        self.expediteur = expediteur
        self.attachements = attachement
        self.destinataires = {'to':to,'cc':cc,'bcc':bcc}


    def raw_message(self):
        """Renders an RFC822 compliant message."""

        rendered_message = cStringIO.StringIO()
        #writer = extended_mime_writer.extended_mime_writer(rendered_message)
        writer=MimeWriter.MimeWriter(rendered_message)
        sender=self.expediteur[0]+' <'+self.expediteur[1]+'>'
        writer.addheader('From', sender)

        for header_key in self.headers.keys():
                if header_key == 'copy-to-folder':
                        continue
                if header_key == 'subject' and self.headers['subject']:
                    suj=self.headers['subject']
                    from cStringIO import StringIO
                    infp = StringIO(suj)
                    outfp = StringIO()
                    quopri.encode(infp, outfp, 1)
                    sujet= outfp.getvalue()
                    sujet=string.replace(sujet, "\n", "")
                    sujet=string.replace(sujet, " ", "_")

                    if string.find(sujet, "?") == -1:
                        sujet="=?iso-8859-1?Q?"+sujet+"?="
                    else:
                        sujet=string.replace(sujet, "?", "=3F")
                        sujet="=?iso-8859-1?Q?"+sujet+"?="
                    writer.addheader(header_key, sujet)
                else:
                    writer.addheader(header_key, self.headers[header_key])

        for address_field in ['to', 'cc']:
            if self.destinataires[address_field]:
                for address in self.destinataires[address_field]:
                    writer.addheader(string.capitalize(address_field),
                                     mimify.mime_encode_header(address))

        if self.getAttachements():
                writer.addheader('Mime-Version', '1.0')

        writer.addheader('X-Mailer', 'WebMailv2 Tice-Universite de Savoie')

        writer.flushheaders()

        if self.getAttachements():
                writer.startmultipartbody('mixed')
                mime_part = writer.nextpart()
                mime_part_writer = mime_part.startbody('text/plain')
                #mimetools.encode(cStringIO.StringIO(self.body),mime_part_writer, 'quoted-printable')
                body = cStringIO.StringIO(self.body)
                encbody = cStringIO.StringIO()
                quopri.encode(body, encbody, 0)
                encbody.seek(0)
                mimetools.copyliteral(encbody, mime_part_writer)

                for attachment in self.getAttachements():
                        mime_part = writer.nextpart()
                        mime_part.addheader('Content-Transfer-Encoding','base64')

                        # ?FIXME?
                        # Let the attachment encoding decide?

                        #tmp_headers = {'content-transfer-encoding':
                                        ##'base64'}
                        ##tmp_headers.update(attachment.headers)
                        mime_part_writer = mime_part.startbody(
                                        attachment.content_type,
                                        [('filename', attachment.filename),('name', attachment.filename)],
                                        1)


                        mime_part_writer.write(attachment.encode())

                else:
                        writer.lastpart()

        else:
                writer._fp.write('Content-Transfer-Encoding: quoted-printable\n')
                
                body_writer = writer.startbody('text/plain', [],{'Content-Transfer-Encoding':'quoted-printable'})

                self.body='\n'+self.body
                body = cStringIO.StringIO(self.body)
                encbody = cStringIO.StringIO()
                quopri.encode(body, encbody, 0)
                encbody.seek(0)

                mimetools.copyliteral(encbody, body_writer)
                #mimetools.encode(body, body_writer, 'quoted-printable')


        return rendered_message.getvalue()


    def setRead(self, value):
        """ set read (Seen) flag"""
        

    def setAnswered(self, folderName):
        """ set Answered flag """

        
    def setDeleted(self, value):
        """ set deleted flag """

    def existAttachement(self):
        """ return true if message have one or more attachment """
        return len(self.attachements)


    def render_body(self, body, version='text'):
        """ replace hexadecimal character with ascii"""

        mess=mimify.mime_decode(body)
        mess=string.replace(mess, "=\r\n", "")
        

        if version=="enriched":
            mess=re.sub(r'<param>([\S]*)</param>', r' ', mess)
            for balise in ['<color>', '</color>', '<bold>', '</bold>', '<bigger>', '</bigger>', '<underline>', '</underline>', '<italic>', '</italic>', '<smaller>', '</smaller>', '<center>', '</center>', '<paraindent>', '</paraindent>', '<excerpt>', '</excerpt>','<fontfamily>', '</fontfamily>']:
                mess=string.replace(mess, balise, '')

        if version=='text' or version=='enriched':
            mess=html_quote(mess)
            mess=re.sub(r'(http://\S*)(&quot;)', r'\1 \2 ', mess)
            mess=re.sub(r'(http://\S*)', r'<a href="\1" target="_blank">\1</a>', mess)
            mess=string.replace(mess, '/a> &quot; &gt;', '/a>&quot; &gt;')
            mess=re.sub(r'(https://\S*)', r'<a href="\1" target="_blank">\1</a>', mess)
	    re.sub(r'(mailto:(\S*)\?subject=(\S*))', r'<a href="edit_message?to=\2&subject=\3">\2</a>', mess)
            mess=re.sub(r'(mailto:(\S*))', r'<a href="edit_message?to=\2">\2</a>', mess)
            mess=re.sub(r'(==.*)(REL|ALT)(.*)(\n|\r)(Content-.*\n|\r){1,3}([\S\s]*)(--=.*)(REL|ALT)(?!--)([\S\s]*)', r'\6', mess)

            
        return mess


    def get_parsed_body(self):
        """ return parsed body """

        try:
            if string.find(string.lower(self.headers['content-type']), 'text/html')!=(-1):
		return self.render_body(self.getBody(), 'html')
            elif string.find(self.headers['content-type'], 'text/enriched')!=(-1):
                return newline_to_br(string.replace(string.replace(self.render_body(self.getBody(), 'enriched'), '  ', '&nbsp; '),'\t','&nbsp;&nbsp;&nbsp;&nbsp;'))
            else:
		return newline_to_br(string.replace(string.replace(self.render_body(self.getBody(), 'text'), '  ', '&nbsp; '),'\t','&nbsp;&nbsp;&nbsp;&nbsp;'))
        except:
            return newline_to_br(string.replace(string.replace(self.render_body(self.getBody(), 'text'), '  ', '&nbsp; '),'\t','&nbsp;&nbsp;&nbsp;&nbsp;'))
